<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 * @author    EveryPay
 * @copyright since 2015 EveryPay
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class EveryPayCustomerReturnModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    public $display_column_left = false;
    private $isIframe;

    public function initContent()
    {
        parent::initContent();

        sleep(3);

        $this->isIframe = Tools::getValue('content_only');
        $orderReference = Tools::getValue('order_reference');
        if (empty($orderReference)) {
            $this->displayFailMessage();
            return;
        }

        if ($this->module->isOrderCreatedBefore()) {
            $orderId = $this->module->getOrderIdByReference($orderReference);
            $createdBefore = true;
        } else {
            $orderId = $this->module->getOrderIdByCartId($orderReference);
            $createdBefore = false;
        }

        $order = new Order($orderId);

        // We cannot find the order, although it should have been created before the payment.
        if (!Validate::isLoadedObject($order) && $createdBefore) {
            $this->displayFailMessage();
            return;
        }

        $paymentStatus = $this->module->getPaymentStatusByReference($orderReference);
        if (empty($paymentStatus)) {
            $this->displayNoStatusMessage();
            return;
        }

        if ($paymentStatus === EveryPay::RETURN_FAILED) {
            if (!$createdBefore) {
                // checkout page
                $redirectUrl = $this->context->link->getPageLink('order', true, null, 'step=1');
            } else {
                // restore cart controller
                $redirectUrl = $this->context->link->getPageLink('order', true, null, "submitReorder&id_order=${orderId}");
            }

            if ($this->isIframe) {
                $this->orderCancelIframeRedirect($redirectUrl, $orderId, $createdBefore);
                return;
            }
            if (!$createdBefore) {
                Tools::redirect($redirectUrl);
            } else {
                $this->orderCancel($redirectUrl, $orderId);
                return;
            }
        }

        $orderState = (int) $order->getCurrentState();
        $orderConfUrl = null;
        $transactionResult = $this->getTransactionResult($orderState);
        $invalidAmount = $transactionResult === EveryPay::RETURN_INVALID_AMOUNT;
        if ($transactionResult === EveryPay::RETURN_SUCCESS) {
            if ($this->isIframe) {
                $orderConfUrl = $this->module->getOrderConfUrl($order);
            } else {
                Tools::redirectLink($this->module->getOrderConfUrl($order));
            }
        } else if ($transactionResult === EveryPay::RETURN_PROCESSING || $invalidAmount) {
            $orderInfoUrl = $this->context->link->getPageLink(
                'order-detail',
                true,
                $order->id_lang,
                array(
                    'id_order' => $order->id
                )
            );
        }

        $this->context->smarty->assign(array(
            'is_iframe' => $this->isIframe,
            'checkout_url' => $checkoutUrl ?? '',
            'order_conf_url' => $orderConfUrl,
            'order_info_url' => $orderInfoUrl ?? '',
            'under_paid' => $invalidAmount,
            'order_reference' => $order->id,
            'transaction_result' => $transactionResult,
        ));

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }

    /**
     * Get the transaction result based on the order state.
     *
     * @param int $orderState
     *
     * @return string
     */
    private function getTransactionResult($orderState)
    {
        switch($orderState) {
            case (int) Configuration::get('PS_OS_PAYMENT'):
                return EveryPay::RETURN_SUCCESS;
            case (int) Configuration::get('PS_OS_EVERYPAY_PROCESSING'):
                return EveryPay::RETURN_PROCESSING;
            case (int) Configuration::get('PS_OS_EVERYPAY'):
                return EveryPay::RETURN_INVALID_AMOUNT;
            default:
                return EveryPay::RETURN_CANCEL;
        }
    }

    /**
     * Display a failure message when the customer manually accesses the endpoint and the order cannot be found.
     *
     * @return void
     *
     * @throws PrestaShopException
     */
    private function displayFailMessage()
    {
        $this->context->smarty->assign(array(
            'transaction_result' => 'fail'
        ));

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }

    /**
     * Message displayed when the Callback URL has not been called.
     *
     * @return void
     *
     * @throws PrestaShopException
     */
    private function displayNoStatusMessage()
    {
        $this->context->smarty->assign(array(
            'transaction_result' => 'no-status'
        ));

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }

    /**
     * Redirect the customer to the checkout page when the order is cancelled and the iframe mode is enabled.
     *
     * @param string $checkoutUrl
     *
     * @return void
     *
     * @throws PrestaShopException
     */
    private function orderCancelIframeRedirect($checkoutUrl, $orderId, $createdBefore)
    {
        $this->context->smarty->assign(array(
            'transaction_result' => 'iframe-cancel',
            'checkout_url' => $checkoutUrl,
            'created_before' => $createdBefore,
            'order_reference' => $orderId,
        ));

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }

    private function orderCancel($checkoutUrl, $orderId)
    {
        $this->context->smarty->assign(array(
            'transaction_result' => 'order-cancel',
            'checkout_url' => $checkoutUrl,
            'order_reference' => $orderId,
            'customer_logged' => $this->context->customer->isLogged(),
        ));

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }
}
