<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 * @author    EveryPay
 * @copyright since 2015 EveryPay
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
include_once(_PS_MODULE_DIR_.'everypay/lib/CardStorageLibrary.php');

class EveryPayReturnModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    public $display_column_left = false;
    private $transactionResult;
    private $paymentInfo;
    private $isLiveMode;
    private $isIframe;

    public function init()
    {
        parent::init();
        $this->isLiveMode = Configuration::get("EVERYPAY_LIVE_MODE");
        $this->paymentInfo = $this->getPaymentInfo();
        $this->transactionResult = $this->paymentInfo['payment_state']; // EveryPay::RETURN_PROCESSING; # for testing processing state
        $this->isIframe = Tools::getValue('content_only');
        if (!$this->module->isOrderCreatedBefore() && $this->transactionResult === EveryPay::RETURN_FAILED) {
            Tools::redirect('index.php?controller=order&step=1');
        } elseif ($this->module->isOrderCreatedBefore() && $this->transactionResult === EveryPay::RETURN_FAILED) {
            $orderId = $this->module->getOrderIdByReference($this->paymentInfo['order_reference']);
            if (isset($orderId)) {
                $order = new Order($orderId);
                // cancel the order in PrestaShop
                $order->setCurrentState(Configuration::get('PS_OS_CANCELED'));
            }
        }
    }

    public function initContent()
    {
        $order = null;
        $orderConfUrl = null;
        $orderInfoUrl = null;
        $paymentSuccessful = $this->transactionResult === EveryPay::RETURN_SUCCESS || $this->transactionResult === EveryPay::RETURN_SETTLED;
        $paymentProcessing = $this->transactionResult === EveryPay::RETURN_PROCESSING;
        $underPaid = false;
        $overPaid = false;
        $paidAmount = 0;
        $overPaidByAmount = 0;
        $initialOrderStatus = 0;
        if (Configuration::get($this->module->formatName('create_order')) === EveryPay::ORDER_BEFORE) {
            $orderId = $this->module->getOrderIdByReference($this->paymentInfo['order_reference']);
        } else {
            do { // creating the order - wrapped in a onetime loop to be able to break out using guard cases (avoiding nested if conditions)
                if (!($paymentSuccessful || $paymentProcessing)) {
                    break;
                    # do not create order
                }

                $orderState = $paymentSuccessful ? Configuration::get('PS_OS_PAYMENT') :
                    ($paymentProcessing ? Configuration::get('PS_OS_EVERYPAY_PROCESSING') : Configuration::get('PS_OS_EVERYPAY'));
                $cartId = (int)$this->paymentInfo['order_reference'];
                $cart = new Cart($cartId);

                $paidAmount = (float)$this->paymentInfo['initial_amount']; // TODO: what if payment is processing and finally gets cancelled?
                $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

                $roundingDecimals = 0;
                $paidAmountRounded = number_format($paidAmount, $roundingDecimals , '.','');
                $totalRounded = number_format($total, $roundingDecimals , '.','');

                # check that cart total was not changed during payment
                $amounts_comparing = bccomp($paidAmountRounded, $totalRounded, $roundingDecimals );
                if (0 > $amounts_comparing) { # if ($paidAmount < $total)
                    $underPaid = true;
                    $orderState = Configuration::get('PS_OS_EVERYPAY');
                    $total = $paidAmount;
                } elseif (0 < $amounts_comparing) {
                    $overPaid = true;
                }

                if (!$cart->orderExists()) {

                    if ((Configuration::get('PS_OS_EVERYPAY_PROCESSING') == $orderState) && true == $overPaid) {
                        $total = 0;
                    }

                    $customer = new Customer($cart->id_customer);
                    $this->module->validateOrder(
                        $cart->id,
                        $orderState,
                        $total,
                        $this->getPaymentMethod(),
                        null,
                        array(),
                        (int)$this->context->currency->id,
                        false,
                        $customer->secure_key
                    );

                    if (true == $overPaid) {
                        $overPaidByAmount = $paidAmount - $total;
                    }

                }
                $orderId = $this->module->getOrderIdByCartId($cart->id);

            } while (0);
        }

        do { // handling the order - wrapped in a onetime loop to be able to break out using guard cases (avoiding nested if conditions)
            if (empty($orderId)) {
                break;
            }

            $order = new Order($orderId);

            if (!Validate::isLoadedObject($order)) {
                break;
            }

            $initialOrderStatus = $order->getCurrentState();

            # add order payment for underpaid order if no payment exists
            if ($underPaid && $paidAmount > 0) { # these are assigned only when order create after mode is used
                # check if no payment exists:
                $hasPayments = false;
                $orderPayments = $order->getOrderPaymentCollection();
                foreach($orderPayments as $orderPayment){
                    $hasPayments = true;
                    break;
                }

                if (!$hasPayments) {

                    $moduleName = $this->module->name;
                    $message = "$moduleName: Amount paid ($paidAmount) was less than order total!";
                    $this->module->insertOrderMessage ($message, $order);

                    if (!$order->addOrderPayment($paidAmount)) {
                        PrestaShopLogger::addLog('EveryPayReturnModuleFrontController::initContent - Cannot save Order Payment', 3, null, 'Order', (int) $orderId, true);

                        // throw new PrestaShopException('Can\'t save Order Payment'); # possible use
                    }
                }
            } elseif ($overPaidByAmount > 0){ # only appears when order is overpaid and is not created yet

                $moduleName = $this->module->name;
                $message = "$moduleName: Amount paid ($paidAmount) was more than order total!";
                $this->module->insertOrderMessage ($message, $order);

                if (!$order->addOrderPayment($overPaidByAmount)) {
                    PrestaShopLogger::addLog('EveryPayReturnModuleFrontController::initContent - Cannot save Order Payment', 3, null, 'Order', (int) $orderId, true);

                    // throw new PrestaShopException('Can\'t save Order Payment'); # possible use
                }
            }

            # update order status
            if ($underPaid) {
                # skip status updating
            } else if ($paymentSuccessful && ($order->getCurrentState() != Configuration::get("PS_OS_PAYMENT"))) {
                $order->setCurrentState(Configuration::get("PS_OS_PAYMENT")); // update status only if not already 'paid'

                if (
                    (Configuration::get("PS_OS_EVERYPAY_PROCESSING") == $initialOrderStatus)
                    && (Configuration::get($this->module->formatName('create_order')) !== EveryPay::ORDER_BEFORE)
                    && $overPaid
                ){
                    # delete (add negative payment equal to) duplicate payment automatically added when changing order state to 'payment_accepted'
                    if (!$order->addOrderPayment((-1) * (float)$cart->getOrderTotal(true, Cart::BOTH))) { # initial cart total must be still equal to '$total' here, but for clarity lets recalc
                        PrestaShopLogger::addLog('EveryPayReturnModuleFrontController::initContent - Cannot save Order Payment', 3, null, 'Order', (int) $orderId, true);

                        // throw new PrestaShopException('Can\'t save Order Payment'); # possible use
                    }
                }

            } elseif ($paymentProcessing &&
                !in_array($order->getCurrentState(), array(Configuration::get("PS_OS_PAYMENT"), Configuration::get("PS_OS_EVERYPAY_PROCESSING")))
            ) {
                $order->setCurrentState(Configuration::get("PS_OS_EVERYPAY_PROCESSING")); // update status only if not already 'paid' or 'processing'
            }

            # save cc data
            if ($paymentSuccessful || $paymentProcessing) {
                if (isset($this->paymentInfo['cc_details']) && isset($this->paymentInfo['cc_details']['token'])) {
                    $this->saveCreditCard($order, $this->paymentInfo['cc_details']);
                }
            }

            # prepare order links
            if ($paymentSuccessful && !$underPaid && !$overPaid) {
                if ($this->isIframe) {
                    $orderConfUrl = $this->module->getOrderConfUrl($order);
                } else {
                    Tools::redirectLink($this->module->getOrderConfUrl($order));
                }
            } elseif ($paymentSuccessful || $paymentProcessing) { # ($paymentSuccessful && ($underPaid || $overPaid)) || $paymentProcessing
                $orderInfoUrl = $this->context->link->getPageLink(
                    'order-detail',
                    true,
                    $order->id_lang,
                    array(
                        'id_order' => $order->id
                    )
                );
            }

        } while (0);

        parent::initContent();
        $this->context->smarty->assign(array(
            'transaction_result' => $this->transactionResult,
            'order_reference' => $order ? $order->id : '',
            'order_conf_url' => $orderConfUrl,
            'is_iframe' => $this->isIframe,
            'order_info_url' => $orderInfoUrl,
            'under_paid' => $underPaid,
            'over_paid' => $overPaid
        ));

        if (EveryPay::DEBUG_LOGGING) {
            $this->module->logError(
                array(
                    'transaction_result' => $this->transactionResult,
                    'order_reference' => $order ? $order->id : '',
                    'order_conf_url' => $orderConfUrl,
                    'is_iframe' => $this->isIframe,
                    'order_info_url' => $orderInfoUrl,
                    'under_paid' => $underPaid,
                    'over_paid' => $overPaid), 'smarty variables: '
            );
        }

        $this->setTemplate('module:everypay/views/templates/front/return.tpl');
    }

    private function saveCreditCard(Order $order, $creditCardInfo)
    {
        $token = new CardStorageLibrary();
        $token->id_customer = $order->id_customer;
        $token->token = $creditCardInfo['token'];
        $token->card_exp_year = $creditCardInfo['year'];
        $token->card_exp_month = $creditCardInfo['month'];
        $cardType = CardStorageLibrary::$cardTypes[$creditCardInfo['type']];
        $token->card_type = $cardType;
        $token->card_no = $creditCardInfo['last_four_digits'];
        if (!$token->doesExist()) {
            $token->save();
        }
    }

    private function getPaymentInfo()
    {
        $payment_reference = Tools::getValue("payment_reference");
        $api_username = $this->isLiveMode ? Configuration::get("EVERYPAY_API_USERNAME") : Configuration::get("EVERYPAY_TEST_API_USERNAME");
        $api_secret = $this->isLiveMode ? Configuration::get("EVERYPAY_SHARED_SECRET") : Configuration::get("EVERYPAY_TEST_SHARED_SECRET");

        $result =  $this->sendGetRequestWithAuth($this->getPaymentResultUrl($payment_reference, $api_username),
            $api_username, $api_secret);
        if (EveryPay::DEBUG_LOGGING) {
            $this->module->logError(array($payment_reference, $result), 'payment info: ');
        }

        try {
            $paymentState = $result['payment_state'];
            $orderReference = $result['order_reference'];
            $savedPaymentStatus = $this->module->getPaymentStatusByReference($orderReference);
            if (empty($savedPaymentStatus)) {
                $this->module->addOrderPaymentStatus($orderReference, $paymentState);
            } else {
                $this->module->updateOrderPaymentStatus($orderReference, $paymentState);
            }
        } catch (Exception $e) {}

        return $result;
    }

    private function getPaymentMethod()
    {
        if (Tools::getValue('payment_method')) {
            return $this->module->getFormattedPaymentMethod(Tools::getValue('payment_method'));
        } else if ($this->paymentInfo['payment_method']) {
            return $this->module->getFormattedPaymentMethod($this->paymentInfo['payment_method']);
        } else {
            return 'missing_payment_method';
        }
    }

    // Not used at the moment because Everypay API now sends payment_reference as GET parameter to customer_url
    private function getCurrentPaymentReference()
    {
        $sql = new DbQuery();
        $sql->select("payment_reference");
        $sql->from("everypay_payments");
        $sql->where("order_reference = ".(int)$this->paymentInfo['order_reference']);

        return Db::getInstance()->executeS($sql)[0]["payment_reference"];
    }

    private function sendGetRequestWithAuth($url, $username, $password)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
            )
        );
        curl_setopt($ch, CURLOPT_USERPWD, $username.":".$password);
        $response = curl_exec($ch);
        curl_close($ch);

        return json_decode($response, true);
    }

    private function getPaymentResultUrl($payment_reference, $api_username)
    {
        $baseUrl = $this->isLiveMode ? "https://pay.every-pay.eu/api/v3/payments/" : "https://igw-demo.every-pay.com/api/v3/payments/";

        return $baseUrl.$payment_reference."?api_username=".$api_username;
    }
}
