/*
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    EveryPay
 *  @copyright since 2010 EveryPay
 *  @license   LICENSE.txt
 */
"use strict";

var EVERYPAY = {};
EVERYPAY.init = function() {
    EVERYPAY.hideDefaultSortOrder();
    EVERYPAY.addSortableId();
    EVERYPAY.sortElements();
    EVERYPAY.buildSortableList();
    EVERYPAY.attachButtonEvent();

    new Sortable(document.querySelector('#sortable'), {
        animation: 150,
    });
}


EVERYPAY.attachButtonEvent = function() {
    const button = document.querySelector('button[name="submitAddconfiguration"]');
    button.addEventListener('click', function() {
        EVERYPAY.updateSortOrder();
    });
}

/**
 * Get the Methods Order input.
 *
 * @returns {*}
 */
EVERYPAY.getMethodsOrderInput = function() {
    return document.getElementById('methods_order');
}

/**
 * Change te Methods Order input's type to hidden.
 */
EVERYPAY.hideDefaultSortOrder = function() {
    EVERYPAY.getMethodsOrderInput().type = 'hidden';
    EVERYPAY.getMethodsOrderInput().parentNode.parentNode.style.display = "none";
}

/**
 * Add the 'sortable' id to the checkboxes parent div.
 */
EVERYPAY.addSortableId = function() {
    const checkboxLabel = document.querySelector('label[for^="enabled_payment_option_"]');
    checkboxLabel.parentNode.parentNode.setAttribute('id', 'sortable');
}

/**
 * Build the sortable list based on the available payment types.
 */
EVERYPAY.buildSortableList = function() {
    const checkboxLabels = document.querySelectorAll('label[for^="enabled_payment_option_"]');

    for (let label of checkboxLabels) {
        const childSpan = document.createElement('span');
        childSpan.classList.add('ui-icon', 'ui-icon-arrowthick-2-n-s');

        const span = document.createElement('span');
        span.classList.add('method-order', 'ui-state-default');
        span.textContent = label.textContent;

        EVERYPAY.removeTextContent(label);

        span.prepend(childSpan);
        label.appendChild(span);
    }
}

EVERYPAY.removeTextContent = function(element) {
    let childNodes = element.childNodes;
    childNodes.forEach(c => c.nodeType === Node.TEXT_NODE && c.remove());
}

/**
 * Get the available payment types and sort them based on the Methods Order input.
 */
EVERYPAY.sortElements = function() {
    const parent = document.querySelector('#sortable');
    const checkboxDivs = parent.querySelectorAll('.checkbox');
    const sortOrder = EVERYPAY.getSortOrder();

    Array.prototype.slice.call(checkboxDivs).sort(function(a, b) {
        const aIndex = a.querySelector('label').htmlFor.replace('enabled_payment_option_', '');
        const bIndex = b.querySelector('label').htmlFor.replace('enabled_payment_option_', '');

        return sortOrder.indexOf(aIndex) - sortOrder.indexOf(bIndex);
    }).forEach(function(el) {
        parent.appendChild(el);
    });

    parent.appendChild(parent.querySelector('p'));
}

/**
 * Get the sort order from the hidden input.
 *
 * @returns {*}
 */
EVERYPAY.getSortOrder = function() {
    return EVERYPAY.getMethodsOrderInput().value.split(',');
}

/**
 * Update the sort order input based on the current sort order.
 */
EVERYPAY.updateSortOrder = function() {
    const methodsOrder = document.querySelectorAll('#sortable label[for^="enabled_payment_option_"]');
    let order = '';
    for (let i = 0; i < methodsOrder.length; i++) {
        order += methodsOrder[i].htmlFor.replace('enabled_payment_option_', '')

        if (i !== methodsOrder.length - 1) {
            order += ',';
        }
    }

    EVERYPAY.getMethodsOrderInput().value = order;
}

$(document).ready(function() {
    EVERYPAY.init();
})
