/*
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    EveryPay
 *  @copyright since 2015 EveryPay
 *  @license   LICENSE.txt
 */
"use strict";
var EVERYPAY = {};
EVERYPAY.id = 'everypay';
EVERYPAY.iframeVisible = 0;
EVERYPAY.managingCards = 0;
EVERYPAY.retry_redirect = window.everypay_checkout_params.retry_redirect;

window.addEventListener('load', function() {
    EVERYPAY.init();
});

EVERYPAY.init = function () {
    EVERYPAY.selectedBanklinkSource = EVERYPAY.getDefaultSelectedBanklinkSource();

    EVERYPAY.changeCard();
    
    EVERYPAY.removeEmptyBankLinkSelectCountryOptions();

    EVERYPAY.setSelectCountry(); // task 17971 - checks and sets selected drop down country as delivery address country

    EVERYPAY.removeLoadingIndicator();

    // task 17972
    EVERYPAY.attachRadioListeners();

    EVERYPAY.attachButtonClickEvent();

    //var termsCheckbox = $('#conditions_to_approve[terms-and-conditions]');
    var termsCheckbox = document.getElementById('conditions_to_approve[terms-and-conditions]');
    attachlisteners(termsCheckbox);
};

EVERYPAY.storeData = function(data) {
    if (!sessionStorage.getItem(EVERYPAY.id)) {
        sessionStorage.setItem(EVERYPAY.id, JSON.stringify({}));
    }
    const storedData = JSON.parse(sessionStorage.getItem(EVERYPAY.id));
    for (let dataKey in data) {
        storedData[dataKey] = data[dataKey];
    }

    sessionStorage.setItem(EVERYPAY.id, JSON.stringify(storedData));
}

// Get the selector for the 'Place Order' button.
EVERYPAY.getPlaceOrderButtonSelector = function() {
    return '#payment-confirmation button, #tc-payment-confirmation button';
}

EVERYPAY.disablePlaceOrderButton = function() {
    document.querySelector(EVERYPAY.getPlaceOrderButtonSelector()).disabled = true;
}

// 'Click' event for the PrestaShop default 'Place Order' button, when the Bank option is selected.
EVERYPAY.attachButtonClickEvent = function() {
    let placeOrderButton = $(EVERYPAY.getPlaceOrderButtonSelector());
    if (!placeOrderButton) {
        return;
    }
    placeOrderButton.on('click', function(e) {
        if (EVERYPAY.isSelectedPaymentMethod('bank')) {
            if (!EVERYPAY.selectedBanklinkSource) {
                EVERYPAY.toggleBankOptionHighlighting(true);
                preventDefaultBehaviour(e);
                return;
            }
        }

        placeOrderIframe(e, 'card');
        placeOrderIframe(e, 'paypal');

        function placeOrderIframe(e, methodId) {
            if (EVERYPAY.isSelectedPaymentMethod(methodId)) {
                preventDefaultBehaviour(e);
                EVERYPAY.disablePlaceOrderButton();
                EVERYPAY.open(methodId);
            }
        }

        function preventDefaultBehaviour(e) {
            e.preventDefault();
            e.stopImmediatePropagation();
        }
    });
}

// Check if the given payment method is selected
EVERYPAY.isSelectedPaymentMethod = function(methodId) {
    return $('.' + EVERYPAY.id + '-form-iframe-' + methodId + ':visible').length > 0;
}

EVERYPAY.getDefaultSelectedBanklinkSource = function() {
    const storedData = JSON.parse(sessionStorage.getItem(EVERYPAY.id));
    if (!storedData || !storedData.selected_provider) {
        return undefined;
    }

    return storedData.selected_provider;
}

EVERYPAY.displayBankLinkPaymentMethods = function (countryCode) {
    if (!countryCode) {
        return;
    }
    const bankLists = document.querySelectorAll('.everypay_banklinks_list');
    bankLists.forEach(function(list) {
        if (list.classList.contains(countryCode)) {
            list.style.display = 'flex';
        } else {
            list.style.display = 'none';
        }
    });
};

EVERYPAY.uncheckBankLinks = function () {
    const links = document.querySelectorAll('.everypay_banklink_container input');
    links.forEach(function(link) {
        link.checked = false;
    });
}

EVERYPAY.removeEmptyBankLinkSelectCountryOptions = function () {
    if(!($('#everypayBanklinkCountrySelect').length)) {
        return;
    }
    $('#everypayBanklinkCountrySelect > option').each(function () {
        const bankContainer = document.querySelectorAll('.everypay_banklinks_list.' + this.value + ' > .everypay_banklink_container');
        if (bankContainer.length === 0) {
            this.remove();
        }
    });
};

// task 17971 sets selected country as cart delivery address country
EVERYPAY.setSelectCountry = function () {
    if(!($('#everypayBanklinkCountrySelect').length) || EVERYPAY.getDefaultSelectedBanklinkSource()) {
        return;
    }
    var customerCountry = 'EE';
    if (everypayIdAddressDelivery === undefined || everypayIdAddressDelivery === null) {
        // pass
    } else {
        customerCountry = (prestashop.customer.addresses[everypayIdAddressDelivery].country_iso ?
            prestashop.customer.addresses[everypayIdAddressDelivery].country_iso : 'EE');
    }
    $('#everypayBanklinkCountrySelect > option').each(function () {
        if (this.value === customerCountry) {
            $('#everypayBanklinkCountrySelect').val(this.value);
            EVERYPAY.processCountrySelectChange();
        }
    });
};

// task 17972
EVERYPAY.enableLinks = function () {
    if(!($('.js-payment-everypay').length)) {
        return;
    }
    $('.js-payment-everypay').each(function () {
        // todo possibly add conditions
        if (1 === 1/*this.value === customerCountry*/) {
            $(this).removeClass('disabled');
        }
    });
};

// task 17972 - this restores everypay payment options visibility after they being hidden by PS if terms checkbox is not checked atm
EVERYPAY.attachRadioListeners = function () {
    var everypayRadioOptions = document.getElementsByName("payment-option");
    everypayRadioOptions.forEach((option) => {
        option.addEventListener("change", async function() {
                await EVERYPAY.sleep(100);
                EVERYPAY.enableLinks();}
            ,false);
    });
};

// task 17972 // this is used to attach listeners to terms checkbox to make banklinks visible on terms status change and to change pay button style
function attachlisteners(element) {
    if (!element) {
        return;
    }
    element.addEventListener("change", async function() {
        await EVERYPAY.sleep(100);
        EVERYPAY.enableLinks();}
    );
}

// task 17972
EVERYPAY.sleep = function (ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
};

// task 17972
EVERYPAY.areTermsAccepted = function () {
    let checkBoxStatus = document.getElementById('conditions_to_approve[terms-and-conditions]');
    if (!checkBoxStatus) {
        return false;
    } else {
        return checkBoxStatus.checked;
    }
};

EVERYPAY.processCountrySelectChange = function () {
    if(!($('#everypayBanklinkCountrySelect').length)) {
        return;
    }
    EVERYPAY.changeSelectedBanklinkSource(undefined); // this resets saved payment source on country change
    let selectedCountry = $('#everypayBanklinkCountrySelect').val();
    EVERYPAY.storeData({selected_country: selectedCountry});
    EVERYPAY.displayBankLinkPaymentMethods(selectedCountry);
    EVERYPAY.uncheckBankLinks();
};

EVERYPAY.startPayment = function (source) {
    if (source !== "card" || source !== "paypal") {
        $("body").css("cursor", "progress");
    }
    var saveCardChecked = ($('.saveCardOption input[type="checkbox"]:checked').length !== 0 ? 1 : 0);
    var iframe = $('.everypayIframe');
    let requestData = {
        saveCard: saveCardChecked,
        payment_method: source
    };
    if (source === 'card') requestData.card_choice = $('input[name="everypayChoice"]:checked').val();
    $.ajax({
        url: iframe.attr("target-url") + "?fc=module&module=everypay&controller=validation&response_type=json",
        dataType: "json",
        data: requestData,
        method: "POST",
        success: function (data) {
            EVERYPAY.order_reference = data.order_reference;
            if (EVERYPAY.retryRedirect) {
                EVERYPAY.retryRedirect = EVERYPAY.retryRedirect.replace("ORDER_REFERENCE", data.order_reference);
            }
            if (source === "card" || source === 'paypal') {
                let paymentLink = undefined;
                if (typeof data.cc_details !== 'undefined') {
                    paymentLink = data.payment_link;
                } else {
                    paymentLink = EVERYPAY.getPaymentMethodInfo(source, data.payment_methods)["payment_link"];
                }
                EVERYPAY.paymentLinkToIframe(paymentLink, source);
            } else {
                window.location = EVERYPAY.getPaymentMethodInfo(source, data.payment_methods)["payment_link"];
            }
        },
        error: function () {
            if (source === "card") {
                var doc = iframe[0].contentWindow.document;
                var $body = $('body', doc);
                $body.html("<p>Cannot get payment page</p>");
                $body.css({"text-align": "center"});
            }
            $("body").css("cursor", "default");
        }
    });
};


EVERYPAY.open = function (paymentMethod) {
    EVERYPAY.startPayment(paymentMethod);
    $('.row').each(function () {
        if ($(this).find('.payment_module').length >= 1 && !$(this).find('.payment_module').hasClass('everypay')) $(this).hide(2000);
    });

    EVERYPAY.openIframe(paymentMethod);

//MANAGE CARDS START
    /*var saveCardChecked = ($('.saveCardOption input[type="checkbox"]:checked').length !== 0 ? 1 : 0);
    var iframe = $('#everypayIframe');
    $.ajax({
       url:iframe.attr("target-url")+"?fc=module&module=everypay&controller=validation&response_type=json",
       dataType:"json",
       data:{card_choice:$('input[name="everypayChoice"]:checked').val(), saveCard:saveCardChecked},
       method:"POST",
       success:function(data) {
             EVERYPAY.order_reference = data.order_reference;
             EVERYPAY.retryRedirect = EVERYPAY.retryRedirect.replace("ORDER_REFERENCE", data.order_reference);
             EVERYPAY.paymentLinkToIframe(EVERYPAY.getPaymentMethodInfo("card", data.payment_methods)["payment_link"]);
       },
        error:function() {
            var doc = iframe[0].contentWindow.document;
            var $body = $('body',doc);
            $body.html("<p>Cannot get payment page</p>");
            $body.css({"text-align": "center"});
        }
    });*/
};

EVERYPAY.getPaymentMethodInfo = function (source, payment_methods) {
    for (var i = 0; i < payment_methods.length; i++) {
        if (payment_methods[i].source === source) return payment_methods[i];
    }
};

EVERYPAY.paymentLinkPageToIframe = function (paymentLink) {
    if ($('#everypayForm').length > 0) $('#everypayForm').remove();
    $('#everypayIframe').parent().append('<form method="post" action="' + paymentLink + '" id="everypayForm"></form>');
    $('#everypayForm').attr("target", "EVERYPAYIFRAME");
    $('#everypayForm').submit();
};

EVERYPAY.paymentLinkToIframe = function (paymentLink, paymentMethod) {
    var $iframe = $("#" + paymentMethod + "-iframe");
    $iframe.attr("src", paymentLink);
};

EVERYPAY.deleteCard = function (obj) {
    if (confirm(window.everypay_checkout_params.everypay_delete_txt)) {
        var link = $(obj).attr('data-remove-card-link');
        $.ajax({
            url: link,
            success: $(obj).parent().remove()
        }).done(function () {
            $('#checkout-payment-step').addClass('-current js-current-step');
            if ($('.everypayOption').length < 1) {
                $('.everypayChoices').find('.everypayManagecards').addClass('hidden');
            }
        });
    }
};

EVERYPAY.setDefault = function (obj) {
    var link = $(obj).attr('data-set-default-link');
    $.ajax({
        url: link
    });
    $(obj).parent().find('.makeDefault').removeClass('manageCardUI').hide();
    $('.everypayOption.default').removeClass('default').find('.makeDefault').addClass('manageCardUI').show();
    $(obj).parent().addClass('default');
};

EVERYPAY.changeCard = function () {
    if ($('input[name="everypayChoice"][value="new"]:checked').length !== 0) {
        $('.saveCardOption').show();
    } else {
        $('.saveCardOption').hide();
    }
};

EVERYPAY.manageCardsToggle = function () {
    if (EVERYPAY.managingCards) {
        $('.manageCardUI').hide();
        EVERYPAY.managingCards = 0;
    } else {
        $('.manageCardUI').show();
        EVERYPAY.managingCards = 1;
    }
};
//MANAGE CARDS END

EVERYPAY.closeIframe = function () {
    EVERYPAY.iframeVisible = 0;
    $("#everypayIframe").show().removeClass('visible iframe-padding').animate({
        height: "0px"
    }, 500);
    $('.everypayUI').show();
};

EVERYPAY.openIframe = function (paymentMethod) {
    var paymentModuleContainerSelector = "." + paymentMethod + "-payment-container";
    $(paymentModuleContainerSelector).css('width', '358px');
    $(paymentModuleContainerSelector).css('height', '570px');
    $("#" + paymentMethod + "-iframe").show().addClass('visible iframe-padding').animate({
        width: "358px",
        height: "650px"
    }, 500);
    $('.everypayUI').hide();
    EVERYPAY.setLoadingIndicatorToIframe(paymentMethod);
    EVERYPAY.iframeVisible = 1;
};

EVERYPAY.setLoadingIndicatorToIframe = function (paymentMethod) {
    var $iframe = $('#' + paymentMethod + '-iframe');
    var doc = $iframe[0].contentWindow.document;
    var $body = $('body', doc);
    $body.html('<img class="loadingIndicator" src="/modules/everypay/views/img/loader.gif"/>');
    $body.css({"text-align": "center"});
};

EVERYPAY.removeLoadingIndicator = function () {
    $(".loadingIndicator").remove();
};

EVERYPAY.reclose = function () {
    if ($('#everypayIframe').attr('order-create-method') != "after") {
        window.location = EVERYPAY.retryRedirect;
    } else {
        $('#everypayRetryButton').hide();
        $('.row').each(function () {
            if ($(this).find('.payment_module').length >= 1 && !$(this).find('.payment_module').hasClass('everypay')) $(this).show(2000);
        });
        EVERYPAY.closeIframe();
        setTimeout("$('.everypayRedirect.button').show();", 2000);
    }
};

EVERYPAY.createQuery = function (data) {
    if ($('#everypayForm').length > 0) $('#everypayForm').remove();
    $('#everypayIframe').parent().append('<form method="post" action="' + data.everypay_url + '" id="everypayForm"></form>');
    var i = 0;
    while (i < data.data.length) {
        $('#everypayForm').append("<input type='hidden' name='" + data.data[i].key + "' value='" + data.data[i].value + "'>");
        i++;
    }
    $('#everypayForm').attr("target", "EVERYPAYIFRAME");
    $('#everypayForm').submit();
};

EVERYPAY.changeSelectedBanklinkSource = function (source) {
    EVERYPAY.storeData({selected_provider: source});
    $('.everypay-bank #bank_selected_provider').val(source);
    this.selectedBanklinkSource = source;
};

// added checking for disabled condition in task 17972
EVERYPAY.startBanklinkPayment = function () {
    if (!EVERYPAY.selectedBanklinkSource) {
        EVERYPAY.toggleBankOptionHighlighting(true);
        return;
    }
    EVERYPAY.startPayment(this.selectedBanklinkSource);
};

// Add/remove highlighting to the bank payment options
EVERYPAY.toggleBankOptionHighlighting = function (error = false) {
    let bankPaymentOptions = document.querySelectorAll('.everypay .banklink_input_label');
    if (bankPaymentOptions.length > 0) {
        for (let option of bankPaymentOptions) {
            if (error) {
                option.classList.add('error');
            } else {
                option.classList.remove('error');
            }
        }
    }
}

EVERYPAY.isApplePayAvailable = function () {
    $.ajax({
        url: "?fc=module&module=everypay&controller=accountinfo",
        dataType: "json",
        method: "GET",
        success: function (data) {
            let paymentMethods = data["payment_methods"];
            for (let i = 0; i < paymentMethods.length; i++) {
                if (paymentMethods[i]["source"] === "card") {
                    if (paymentMethods[i]["applepay_custom_available"] === true) {
                        if (window.ApplePaySession && ApplePaySession.canMakePayments()) {
                            let applePayButton = document.getElementById('apple-pay-button');
                            if (!applePayButton.onclick) {
                                applePayButton.addEventListener('click', async function () {
                                    await EVERYPAY.applePayChangeCursor();
                                    setTimeout(function() {
                                        EVERYPAY.applePayClick();
                                    }, 500);
                                });
                            }
                        } else {
                            EVERYPAY.removeApplePayButton();
                        }
                    } else {
                        EVERYPAY.removeApplePayButton();
                    }
                }
            }
        },
        error: function () {
            EVERYPAY.removeApplePayButton();
        }
    });
};

EVERYPAY.applePayChangeCursor = async function () {
    let applePayButton = document.getElementById('apple-pay-button');
    document.body.style.cursor = 'progress';
    applePayButton.style.cursor = 'progress';
    applePayButton.disabled = true;
};

EVERYPAY.removeApplePayButton = function () {
    let applePayButton = document.getElementById('apple-pay-button');
    let previousSibling = applePayButton.parentNode.previousElementSibling;
    if (previousSibling) {
        previousSibling.remove();
    }
    applePayButton.parentElement.remove();
};

EVERYPAY.applePayClick = function () {
    let applePayButton = document.getElementById('apple-pay-button');
    if (!EVERYPAY.areTermsAccepted()) {
        document.body.style.cursor = 'auto';
        applePayButton.style.cursor = 'pointer';
        applePayButton.disabled = false;
        return;
    }
    let requestData = {
        payment_method: 'apple_pay'
    };

    try {
        let oneoff = $.ajax({
            url: "/?fc=module&module=everypay&controller=validation&response_type=json",
            dataType: "json",
            data: requestData,
            method: "POST",
            async: false,
        });
        EVERYPAY.startApplePaySession(oneoff.responseJSON);
    } catch (error) {
        document.body.style.cursor = 'auto';
        applePayButton.style.cursor = 'pointer';
        applePayButton.disabled = false;
        return;
    }

    setTimeout(function() {
        document.body.style.cursor = 'auto';
        applePayButton.style.cursor = 'pointer';
        applePayButton.disabled = false;
    }, 1500);
};

EVERYPAY.startApplePaySession = function (oneoff) {
    if (window.ApplePaySession && ApplePaySession.canMakePayments()) {
        let store_name = document.getElementById('store-name').value;
        let applePayClient = ApplePayClient.initialize({
            access_token: oneoff.mobile_access_token,
            api_username: oneoff.api_username,
            merchant_id:  oneoff.applepay_merchant_identifier,
            account_name: oneoff.account_name,
        });
        applePayClient.createApplePayPayment({
            payment_reference: oneoff.payment_reference,
            country_code: oneoff.descriptor_country,
            currency_code: oneoff.currency,
            payment_link: oneoff.payment_link,
            total: {
                label: store_name,
                amount: oneoff.initial_amount,
            }
        });
    } else {
        return;
    }
};

var shrinkIframe = function (iframe, iframe_data) {
    iframe.css(iframe_data);
    jQuery("#dimmed_background_box").remove();
};
var expandIframe = function () {
    var iframe_data = {
        position: iframe.attr("position") || "static",
        top: iframe.position().top,
        left: iframe.position().left,
        width: iframe.width(),
        height: iframe.height(),
        zIndex: iframe.attr("zIndex"),
        marginLeft: iframe.attr("marginLeft"),
        marginRight: iframe.attr("marginRight")
    };

    jQuery('body').append("<div id='dimmed_background_box'></div>");
    jQuery('#dimmed_background_box').css({
        height: '100%',
        width: '100%',
        position: 'fixed',
        top: 0,
        left: 0,
        zIndex: 9998,
        backgroundColor: '#000000',
        opacity: 0.5
    });

    var window_height = jQuery(window).height();
    var window_width = jQuery(window).width();

    if (window_width < 960) {
        iframe.css({height: window_height, width: window_width, top: 0});
    } else {
        iframe.css({height: 640, width: 960, top: (window_height - 640) / 2});
    }
    iframe.css({position: 'fixed', zIndex: 9999, margin: 'auto'});
    return iframe_data;
};

var shrinked_iframe_data;
//var iframe = $('#everypayIframe'); // iframe selector should be used

window.addEventListener('message', function (event) {
    if (!event.data){
        return;
    }

    var message = null;

    if (typeof event.data === 'object'){
        message = event.data;
    } else {
        message = JSON.parse(event.data);
    }

    if (!message){
        return;
    }

    /*
    1. An "expand" message is sent from the iframe page when 3D secure page is going to be displayed.
        The size of the iframe should be adjusted to hold 3D secure page
    2. A "shrink" message is sent from the iframe page when a user has provided authorisation details on the 3D secure page.
        The size of the iframe should be set to the initial values
    */
    if (message.resize_iframe == "expand") {
        shrinked_iframe_data = expandIframe(iframe);
    } else if (message.resize_iframe == "shrink") {
        shrinkIframe(iframe, shrinked_iframe_data);
    }

    if (message.transaction_result) {
        if (message.transaction_result == "completed") {
            window.location = $('#everypayIframe').attr("target-url") + "?fc=module&module=everypay&controller=return&order_reference=" + EVERYPAY.order_reference + "&transaction_result=" + message.transaction_result;
        } else {
            if (EVERYPAY.iframeVisible) {
                $('#everypayRetryButton').show();
            } else {
                EVERYPAY.openIframe();
                $('#everypayRetryButton').show();
            }
        }
    }
}, false);

function var_dump(obj, alert) {
    var out = '';
    for (var i in obj) {
        out += i + ": " + obj[i] + "\n";
    }
    if (alert) alert(out);
    return out;
}
